import React, { useState, useEffect, useImperativeHandle } from 'react'
import styled from 'styled-components'
import Sortable from 'sortablejs'
import arrowImgUrl from '../style/img/arrow.png'
import syncImgUrl from '../style/img/sync.png'
import warningImgUrl from '../style/img/warning.png'
import deleteImgUrl from '../style/img/delete.png'
import { getTableSchema } from '../services'
import { Form, Select } from 'antd'

const StepThreeDiv = styled.div`
  .title {
    font-weight: 600;
    border-left: 3px solid #1881da;
    padding-left: 12px;
  }
  .tip {
    font-size: 12px;
    font-weight: 400;
    color: #bbbbbb;
    line-height: 17px;
    padding-left: 80px;
    margin: 30px 0 30px;
  }
  .table__wrapper {
    padding-left: 80px;
    margin-bottom: 30px;
    display: flex;
  }
  .table__source {
    border-radius: 2px;
    border: 1px solid #c8d3e9;
    width: 400px;
  }
  .table__title {
    border-bottom: 1px solid #c8d3e9;
    height: 50px;
    display: flex;
    align-items: center;
    justify-content: space-between;
    padding: 0 20px;
  }
  .table__content {
    height: 500px;
    display: felx;
    justify-content: space-between;
    padding: 10px 20px;
    overflow-y: scroll;
  }
  .field__wrapper {
    background: #ebeff3;
    padding: 8px 10px;
    width: 140px;
    font-size: 12px;
    font-weight: 400;
    line-height: 17px;
  }
  .field__sync {
    background: #eef5fe;
  }
  .arrow {
    padding: 0 20px;
    display: flex;
    align-items: center;
  }
  .sync__img {
    display: flex;
    align-items: center;
    height: 33px;
    margin-bottom: 1em;
  }
  .mapping__content {
    height: 500px;
    padding: 10px 20px;
    overflow-y: scroll;
  }
  .mapping__item {
    display: flex;
    justify-content: space-between;
    align-items: center;
    margin-bottom: 1em;
  }
`

export default function StepThree({
  onRef,
  ruleForm,
  drawDataForm,
  loadDataForm,
}) {
  // 映射管理
  const [syncMappings, setSyncMappings] = useState([])
  const [datasourceFields, setDatasourceFields] = useState([])
  const [syncFields, setSyncFields] = useState([])

  // 暴露同步map
  useImperativeHandle(onRef, () => {
    return {
      syncMappings,
    }
  })

  // 拖拽
  const drapDrop = () => {
    const datasourceEl = document.querySelector('.sortableDatasource')
    const datasourceSortable = new Sortable(datasourceEl, {
      animation: 150,
      onEnd: evt => {
        const { oldIndex, newIndex } = evt
        const changedItem = datasourceFields.splice(oldIndex, 1)[0]
        datasourceFields.splice(newIndex, 0, changedItem)
        setDatasourceFields([...datasourceFields])
      },
    })
    const syncEl = document.querySelector('.sortableSync')
    const syncSortable = new Sortable(syncEl, {
      animation: 150,
      onEnd: evt => {
        const { oldIndex, newIndex } = evt
        const changedItem = syncFields.splice(oldIndex, 1)[0]
        syncFields.splice(newIndex, 0, changedItem)
        setSyncFields([...syncFields])
      },
    })
  }
  const fetchDatasource = async (id, table_name) => {
    const { data } = await getTableSchema({ id, table_name })
    if (data.code === 200) {
      const list = formatData(data.data)
      setDatasourceFields(list)
      console.log('222', list, datasourceFields)
    }
  }

  const fetchSyncData = async (id, table_name) => {
    const { data } = await getTableSchema({ id, table_name })
    if (data.code === 200) {
      const list = formatData(data.data)
      setSyncFields(list)
    }
  }
  const formatData = data => {
    const list = data.map(item => {
      const splitData = item.split(':')
      return {
        key: splitData[0],
        id: splitData[0],
        value: splitData[1],
        type: splitData[2],
      }
    })
    return list
  }

  useEffect(() => {
    const ds_id = drawDataForm.getFieldValue('datasource_name')
    const table_name = drawDataForm.getFieldValue('datasource_table')
    if (ds_id && table_name) {
      fetchDatasource(ds_id, table_name)
    }
  }, [drawDataForm])

  useEffect(() => {
    const ds_id = loadDataForm.getFieldValue('datasource_name')
    const table_name = loadDataForm.getFieldValue('datasource_table')
    if (ds_id && table_name) {
      fetchSyncData(ds_id, table_name)
    }
  }, [loadDataForm])

  useEffect(() => {
    drapDrop()
  }, [])

  const syncFieldData = index => {
    setSyncMappings([
      ...syncMappings,
      {
        datasourceField: datasourceFields[index].value,
        syncField: syncFields[index].value,
      },
    ])
  }
  const deleteMapping = mapping => {
    setSyncMappings(syncMappings.filter(item => item !== mapping))
  }
  return (
    <StepThreeDiv>
      <p className="title">配置转换规则</p>
      <p className="tip">拖拽提取源数据表中的字段，与加载源数据表</p>
      <div className="table__wrapper">
        <div className="table__source">
          <div className="table__title">
            <span>
              提取源：{drawDataForm.getFieldValue('datasource_table')}
            </span>
            <span>
              加载源：{loadDataForm.getFieldValue('datasource_table')}
            </span>
          </div>
          <div className="table__content">
            <div className="sortableDatasource">
              {datasourceFields.map(item => {
                return (
                  <p className="field__wrapper field__sync" key={item.key}>
                    {item.value}
                  </p>
                )
              })}
            </div>
            <div>
              {datasourceFields.map((item, index) => {
                const sameType =
                  index > syncFields.length - 1
                    ? false
                    : datasourceFields[index].type === syncFields[index].type
                return (
                  <div className="sync__img" key={item.key}>
                    <img
                      src={sameType ? syncImgUrl : warningImgUrl}
                      alt=""
                      onClick={() => {
                        if (sameType) syncFieldData(index)
                      }}
                      style={{
                        width: '20px',
                        height: '20px',
                        cursor: sameType ? 'pointer' : 'no-drop',
                      }}
                    />
                  </div>
                )
              })}
            </div>
            <div className="sortableSync">
              {syncFields.map(item => {
                return (
                  <p className="field__wrapper" key={item.key}>
                    {item.value}
                  </p>
                )
              })}
            </div>
          </div>
        </div>
        <div className="arrow">
          <img
            src={arrowImgUrl}
            alt=""
            style={{ width: '20px', height: '20px' }}
          />
        </div>
        <div className="table__source">
          <div className="table__title">
            <span>映射管理</span>
            <span
              onClick={() => {
                setSyncMappings([])
              }}
              style={{
                color: '#7D7D7D',
                cursor: 'pointer',
              }}>
              清空
            </span>
          </div>
          <div className="mapping__content">
            {syncMappings.map((item, index) => {
              return (
                <div className="mapping__item" key={index}>
                  <span style={{ width: '30px' }}>{index + 1}</span>
                  <span className="field__wrapper">{item.datasourceField}</span>
                  <img
                    src={syncImgUrl}
                    alt=""
                    style={{ width: '20px', height: '20px' }}
                  />
                  <span className="field__wrapper">{item.syncField}</span>
                  <img
                    src={deleteImgUrl}
                    alt=""
                    onClick={() => deleteMapping(item)}
                    style={{ width: '15px', height: '15px', cursor: 'pointer' }}
                  />
                </div>
              )
            })}
          </div>
        </div>
      </div>
      <Form
        name="ruleForm"
        form={ruleForm}
        labelCol={{
          span: 2,
        }}
        wrapperCol={{
          span: 6,
        }}>
        <Form.Item
          label="写入模式"
          name="write_mode"
          rules={[
            {
              required: true,
              message: '请选择模式',
            },
          ]}>
          <Select
            options={[
              { value: 'add', label: '追加' },
              { value: 'cover', label: '覆盖' },
            ]}
            allowClear
          />
        </Form.Item>
      </Form>
    </StepThreeDiv>
  )
}
